@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# Project   : Cuisine - Functions to write Fabric recipies
+# Project   : Cuisine - Functions to write Fabric recipes
 # -----------------------------------------------------------------------------
 # Author    : Sebastien Pierre                            <sebastien@ffctn.com>
 # Author    : Thierry Stiegler   (gentoo port)     <thierry.stiegler@gmail.com>
@@ -13,7 +13,7 @@
 
 """
 `cuisine` makes it easy to write automatic server installation
-and configuration recipies by wrapping common administrative tasks
+and configuration recipes by wrapping common administrative tasks
 (installing packages, creating users and groups) in Python
 functions.
 
@@ -38,20 +38,18 @@ See also:
 :license:   BSD, see LICENSE for more details.
 """
 
-import base64, bz2, crypt, hashlib, os, random, sys, re, string, tempfile, subprocess, types
+import base64, bz2, crypt, hashlib, os, random, sys, re, string, tempfile, subprocess, types, functools
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION     = "0.2.2"
+VERSION     = "0.2.3"
 
 RE_SPACES   = re.compile("[\s\t]+")
 MAC_EOL     = "\n"
 UNIX_EOL    = "\n"
 WINDOWS_EOL = "\r\n"
 # FIXME: MODE should be in the fabric env, as this is definitely not thread-safe
-MODE_USER   = "user"
 MODE_LOCAL  = False
-MODE_SUDO   = "sudo"
-MODE        = MODE_USER
+MODE_SUDO   = False
 DEFAULT_OPTIONS = dict(
 	package="apt"
 )
@@ -69,11 +67,11 @@ def mode_local():
 	global MODE_LOCAL
 	if MODE_LOCAL is False:
 		def custom_run( cmd ):
-			global MODE
-			if MODE == "sudo":
-				return os.popen(cmd).read()[:-1]
-			else:
+			global MODE_SUDO
+			if MODE_SUDO:
 				return os.popen("sudo " + cmd).read()[:-1]
+			else:
+				return os.popen(cmd).read()[:-1]
 		def custom_sudo( cmd ):
 			return os.popen("sudo " + cmd).read()[:-1]
 		module   = sys.modules[__name__]
@@ -91,6 +89,7 @@ def mode_remote():
 	the effect of calling `mode_local()`."""
 	global MODE_LOCAL
 	if not (MODE_LOCAL is False):
+		module = sys.modules[__name__]
 		setattr(module, "run",  MODE_LOCAL[0])
 		setattr(module, "sudo", MODE_LOCAL[1])
 		MODE_LOCAL = False
@@ -103,31 +102,31 @@ class mode_user(object):
 	"""Cuisine functions will be executed as the current user."""
 
 	def __init__(self):
-		global MODE
-		self._old_mode = MODE
-		MODE = MODE_USER
+		global MODE_SUDO
+		self._old_mode = MODE_SUDO
+		MODE_SUDO = False
 
 	def __enter__(self):
 		pass
 
 	def __exit__(self, *args, **kws):
-		global MODE
-		MODE = self._old_mode
+		global MODE_SUDO
+		MODE_SUDO = self._old_mode
 
 class mode_sudo(object):
 	"""Cuisine functions will be executed with sudo."""
 
 	def __init__(self):
-		global MODE
-		self._old_mode = MODE
-		MODE = MODE_SUDO
+		global MODE_SUDO
+		self._old_mode = MODE_SUDO
+		MODE_SUDO = True
 
 	def __enter__(self):
 		pass
 
 	def __exit__(self, *args, **kws):
-		global MODE
-		MODE = self._old_mode
+		global MODE_SUDO
+		MODE_SUDO = self._old_mode
 
 # =============================================================================
 #
@@ -151,9 +150,9 @@ def select_package( option=None ):
 
 def run(*args, **kwargs):
 	"""A wrapper to Fabric's run/sudo commands, using the
-	'cuisine.MODE' global to tell wether the command should be run as
+	'cuisine.MODE' global to tell whether the command should be run as
 	regular user or sudo."""
-	if MODE == MODE_SUDO:
+	if MODE_SUDO:
 		return fabric.api.sudo(*args, **kwargs)
 	else:
 		return fabric.api.run(*args, **kwargs)
@@ -168,7 +167,7 @@ def run_local(command):
 
 def sudo(*args, **kwargs):
 	"""A wrapper to Fabric's run/sudo commands, using the
-	'cuisine.MODE' global to tell wether the command should be run as
+	'cuisine.MODE_SUDO' global to tell whether the command should be run as
 	regular user or sudo."""
 	return fabric.api.sudo(*args, **kwargs)
 
@@ -224,8 +223,7 @@ def dispatch(prefix=None):
 			else:
 				raise Exception("Function variant not defined: " + function_name)
 		# We copy name and docstring
-		wrapper.__name__ = function.__name__
-		wrapper.__doc__  = function.__doc__
+		functools.update_wrapper(wrapper, function)
 		return wrapper
 	if type(prefix) == types.FunctionType:
 		return dispatch_wrapper(prefix, None)
@@ -370,7 +368,7 @@ def file_attribs_get(location):
 	else:
 		return None
 
-def file_write(location, content, mode=None, owner=None, group=None):
+def file_write(location, content, mode=None, owner=None, group=None, sudo=None):
 	"""Writes the given content to the file at the given remote
 	location, optionally setting mode/owner/group."""
 	# FIXME: Big files are never transferred properly!
@@ -380,9 +378,9 @@ def file_write(location, content, mode=None, owner=None, group=None):
 	os.write(fd, content)
 	# Upload the content if necessary
 	if not file_exists(location) or sig != file_sha256(location):
-		if MODE == MODE_SUDO: mode = MODE_SUDO
+		if MODE_SUDO: sudo = MODE_SUDO
 		try:
-			fabric.operations.put(local_path, location, use_sudo=(mode == MODE_SUDO))
+			fabric.operations.put(local_path, location, use_sudo=sudo)
 		except Exception, e:
 			print "cuisine.file_write exception:"
 	# Remove the local temp file
@@ -400,7 +398,7 @@ def file_ensure(location, mode=None, owner=None, group=None, recursive=False):
 	else:
 		file_write(location,"",mode=mode,owner=owner,group=group)
 
-def file_upload(remote, local):
+def file_upload(remote, local, sudo=None):
 	"""Uploads the local file to the remote location only if the remote location does not
 	exists or the content are different."""
 	# FIXME: Big files are never transferred properly!
@@ -408,8 +406,9 @@ def file_upload(remote, local):
 	content = f.read()
 	f.close()
 	sig     = hashlib.sha256(content).hexdigest()
+	if MODE_SUDO: sudo = MODE_SUDO
 	if not file_exists(remote) or sig != file_sha256(remote):
-		fabric.operations.put(local, remote, use_sudo=(MODE == MODE_SUDO))
+		fabric.operations.put(local, remote, use_sudo=sudo)
 
 def file_update(location, updater=lambda x: x):
 	"""Updates the content of the given by passing the existing
@@ -498,6 +497,9 @@ def package_upgrade():
 def package_update(package=None):
 	"""Updates the package database (when no argument) or update the package
 	or list of packages given as argument."""
+@dispatch
+def package_update(package=None):
+	"""Upgrade the system."""
 
 @dispatch
 def package_install(package, update=False):
@@ -528,6 +530,9 @@ def package_update_apt(package=None):
 			package = " ".join(package)
 		sudo("apt-get --yes upgrade " + package)
 
+def package_upgrade_apt(package=None):
+	sudo("apt-get --yes upgrade")
+
 def package_install_apt(package, update=False):
 	if update:
 		sudo("apt-get --yes update")
@@ -573,6 +578,11 @@ def command_ensure(command, package=None):
 #
 # =============================================================================
 
+def user_passwd(name, passwd):
+	"""Sets the given user password."""
+	encoded_password = base64.b64encode("%s:%s" % (name, passwd))
+	sudo("echo %s | base64 --decode | chpasswd" % encoded_password)
+
 def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
 				uid_min=None, uid_max=None):
 	"""Creates the user with the given name, optionally giving a
@@ -595,8 +605,7 @@ def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
 		options.append("-K UID_MAX='%s'" % (uid_max))
 	sudo("useradd %s '%s'" % (" ".join(options), name))
 	if passwd:
-		sudo("echo %s:%s | chpasswd" % (name, passwd))
-
+		user_passwd(name,passwd)
 
 def user_check(name):
 	"""Checks if there is a user defined with the given name,
@@ -608,7 +617,7 @@ def user_check(name):
 	results = {}
 	if d:
 		d = d.split(":")
-		assert len(d) >= 7, "/etc/passwd entry is expected to have at least 7 fields, got %s in: %s" % (len(d), d.join(":"))
+		assert len(d) >= 7, "/etc/passwd entry is expected to have at least 7 fields, got %s in: %s" % (len(d), ":".join(d))
 		results = dict(name=d[0], uid=d[2], gid=d[3], home=d[5], shell=d[6])
 	if s:
 		results['passwd'] = s
@@ -636,8 +645,7 @@ def user_ensure(name, passwd=None, home=None, uid=None, gid=None, shell=None):
 		if options:
 			sudo("usermod %s '%s'" % (" ".join(options), name))
 		if passwd:
-			encoded_password = base64.b64encode("%s:%s" % (name, passwd))
-			sudo("echo %s | base64 --decode | chpasswd" % (encoded_password,name, passwd))
+			user_passwd(name, passwd)
 
 # =============================================================================
 #
@@ -736,18 +744,19 @@ def ssh_authorize(user, key):
 			return True
 	else:
 		# Make sure that .ssh directory exists, see #42
-		dir_ensure(os.path.dirname(keyf), user=user, group=user, mode="700")
-		file_write(keyf, key,             user=user, group=user, mode="600")
+		dir_ensure(os.path.dirname(keyf), owner=user, group=user, mode="700")
+		file_write(keyf, key,             owner=user, group=user, mode="600")
 		return False
 
 def upstart_ensure(name):
 	"""Ensures that the given upstart service is running, restarting
 	it if necessary."""
-	status = sudo("service %s status" % name)
-	if status.find("is running") >= 0 or status.find("/running") >= 0:
-		sudo("service %s restart" % name)
-	else:
+	with fabric.api.settings(warn_only=True):
+		status = sudo("service %s status" % name)
+	if status.failed:
 		sudo("service %s start" % name)
+	else:
+		sudo("service %s restart" % name)
 
 def system_uuid_alias_add():
 	"""Adds system UUID alias to /etc/hosts.
